@@ -20,68 +20,63 @@ function(x, offset = TRUE, intercept = FALSE, ...) {
 		offs <- sapply(variables[attr(x, "offset")], deparse)
 	} else offs <- NULL
 
-	ret <- attr(x, "term.labels")
+	ans <- attr(x, "term.labels")
 
 	# Get term names, with higher order term components arranged alphabetically
-	if (length(ret) > 0L) {
+	if (length(ans) > 0L) {
 		factors <- attr(x, "factors")
 		factors <- factors[order(rownames(factors)), , drop = FALSE]
 		v <- rownames(factors)
-		ret <- apply(factors != 0L, 2L, function(x) paste(v[x], collapse = ":"))
+		ans <- apply(factors != 0L, 2L, function(x) paste0(v[x], collapse = ":"))
 	}
 
 	# Leave out random terms (lmer type)
 	#ran <- attr(x, "variables")[-1][-c(attr(x, "offset"), attr(x, "response"))]
-	ran <- variables
-	ran <- as.character(ran[sapply(ran,
-		function(x) length(x) == 3L && x[[1L]] == "|")])
-	ifx <- !(ret %in% ran)
+	ran <- as.character(variables[vapply(variables, function(x) length(x) == 3L && x[[1L]] == "|", TRUE)])
+	ifx <- !(ans %in% ran)
 
-	ret <- ret[ifx] # ifx - indexes of fixed terms
-	#retUnsorted <- ret
+	ans <- ans[ifx] # ifx - indexes of fixed terms
+	#retUnsorted <- ans
 
 	# finally, sort by term order and then alphabetically
-	#ret <- unname(ret[order(attr(x, "order")[ifx], ret)])
-	ord <- order(attr(x, "order")[ifx], gsub("I\\((.*)\\)", "\\1", ret))
-	ret <- unname(ret[ord])
+	#ans <- unname(ans[order(attr(x, "order")[ifx], ans)])
+	ord <- order(attr(x, "order")[ifx], gsub("I\\((.*)\\)", "\\1", ans))
+	ans <- unname(ans[ord])
 
-	deps <- if (length(ret) > 0L) termdepmat(reformulate(ret)) else
+	deps <- if (length(ans) > 0L) termdepmat(reformulate(ans)) else
 		matrix(FALSE, 0L, 0L)
 		
-	dimnames(deps) <- list(ret, ret)
+	dimnames(deps) <- list(ans, ans)
 	diag(deps) <- NA
 	
 	if(intercept && attr(x, "intercept")) {
-		ret <- c(interceptLabel, ret)
-		ord <- c(1, ord + 1L)
+		ans <- c(interceptLabel, ans)
+		ord <- c(1L, ord + 1L)
 	}
 
 	if (!is.null(offs[1L])) {
 		if (offset) {
-			ret <- c(ret, offs)
+			ans <- c(ans, offs)
 			ord <- c(ord, length(ord) + 1L)
 		}
-		attr(ret, "offset") <- offs
+		attr(ans, "offset") <- offs
 	}
-	attr(ret, "intercept") <- attr(x, "intercept")
-	attr(ret, "interceptLabel") <- interceptLabel
-	
+	attr(ans, "intercept") <- attr(x, "intercept")
+	attr(ans, "interceptLabel") <- interceptLabel
 
 	if (length(ran) > 0L) {
-		attr(ret, "random.terms") <- ran
+		attr(ans, "random.terms") <- ran
 		f.random <- reformulate(c(".", paste0("(", ran, ")")), response = ".")
 		environment(f.random) <- environment(x)
-		attr(ret, "random") <- f.random
+		attr(ans, "random") <- f.random
 	}
 
 	response <- attr(x, "response")
 	response <- if(response == 0L) NULL else variables[[response]]
-	attr(ret, "response") <- response
-	attr(ret, "order") <- order(ord)
-	attr(ret, "deps") <- deps
-
-
-	return(ret)
+	attr(ans, "response") <- response
+	attr(ans, "order") <- order(ord)
+	attr(ans, "deps") <- deps
+	ans
 }
 
 `getAllTerms.formula` <-
@@ -112,7 +107,7 @@ function(x, ...) {
 # `getAllTerms.glmer` <- # For backwards compatibility
 # `getAllTerms.lmer` <-  # with older versions of lme4
 `getAllTerms.mer` <-
-function(x, ...) getAllTerms(.xget("lme4", "formula")(x), ...)
+function(x, ...) getAllTerms(getFrom("lme4", "formula")(x), ...)
 
 # Apparently there is no (explicit) intercept in coxph, but 'terms' gives
 # attr(,"intercept") == 1.
@@ -125,7 +120,6 @@ function(x, ...) getAllTerms(.xget("lme4", "formula")(x), ...)
 
 `getAllTerms.glmmML` <- function (x, ...) {
 	ret <- getAllTerms.terms(terms(x), ...)
-	#attr(ret, "random.terms") <-  deparse(call("|", 1,  x$call$cluster))
 	attr(ret, "random.terms") <-  paste("1 |",  x$call$cluster)
 	return(ret)
 }
@@ -140,46 +134,101 @@ function(x, ...) getAllTerms(.xget("lme4", "formula")(x), ...)
 #	else getAllTerms.formula(f, intercept = intercept)
 #}
 
+split_formula_by_bar <- function(f) {
+	n <- length(f)
+	ans <- if(length(f[[n]]) != 1L && f[[n]][[1L]] == "|") {
+		f1 <- vector("list", 2L)
+		for(i in 1L:2L) {
+			f1[[i]] <- f
+			f1[[i]][[n]] <- f[[n]][[i+ 1]]
+		}
+		f1
+	} else list(f)
+	ans
+}
+
+
 `getAllTerms.hurdle` <- 
 `getAllTerms.zeroinfl` <-
 function(x, intercept = FALSE, ...) {
-	f <- formula(x)
-	if(length(f[[3L]]) != 1L && f[[3L]][[1L]] == "|"){
-		f1 <- call("~", f[[2L]], f[[3L]][[2L]])
-		f2 <- call("~", f[[3L]][[3L]])
-	} else {
-		f1 <- f
-		f2 <- NULL
-	}
-	fs <- lapply(lapply(c(f1, f2), terms.formula, data = eval(x$call$data)),
+
+	formList <- split_formula_by_bar(formula(x))
+	formList <- lapply(lapply(formList, terms.formula, data = eval(x$call$data)),
 		formula)
-	z <- lapply(fs, getAllTerms, intercept = TRUE)
+	z <- lapply(formList, getAllTerms, intercept = TRUE)
 	
+	if(oneform <- length(formList) == 1L) z <- c(z, z)
+
 	deps <- termdepmat_combine(lapply(z, attr, "deps"))
 
 	ord <- unlist(lapply(z, attr, "order"))
 	n <- sapply(z, length)
 	if(length(z) > 1L) ord[-j] <- ord[-(j <- seq_len(n[1L]))] + n[1L]
+		
 	zz <- unlist(z)
-	Ints <- which(zz == "(Intercept)")
-	#zz[Ints] <- "1"
-	#zz <- paste0(rep(c("count", "zero")[seq_along(z)], sapply(z, length)),
-		#"(", zz, ")")
+	interceptIdx <- zz == "(Intercept)"
+	offsetIdx <- match(zz, unique(unlist(lapply(z, attr,"offset"))), nomatch = 0) != 0
+	termIdx <- !(offsetIdx | interceptIdx)
+
 	zz <- paste0(rep(c("count", "zero")[seq_along(z)], sapply(z, length)),
 				 "_", zz)
 	
-	dimnames(deps) <- list(zz[-Ints], zz[-Ints])
+	dimnames(deps) <- list(zz[termIdx], zz[termIdx])
 	
-	ret <- if(!intercept) zz[-Ints] else zz
-	attr(ret, "intercept") <- pmin(Ints, 1)
-	attr(ret, "interceptLabel") <- zz[Ints]
+	if(oneform) { # dependency of count_X and zero_X
+		k <- length(zz[termIdx]) / 2
+		deps[c(seq(k + 1L, by = 2L * k + 1L, length.out = k),
+			   seq((2L * k * k) + 1L, by = 2L * k + 1L, length.out = k))] <- TRUE
+	}
+	
+	ret <- if(!intercept) zz[!interceptIdx] else zz
+	if(any(offsetIdx)) attr(ret, "offset") <- zz[offsetIdx]
+	attr(ret, "intercept") <- pmin(which(interceptIdx), 1)
+	attr(ret, "interceptLabel") <- zz[interceptIdx]
 	attr(ret, "response") <- attr(z[[1L]], "response")
-	attr(ret, "order") <- if(!intercept) order(ord[-Ints]) else ord
+	attr(ret, "order") <- if(!intercept) order(ord[!interceptIdx]) else ord
 	attr(ret, "deps") <- deps
 	ret
-	
 }
 
+## TODO: test with offsets
+`getAllTerms.betareg` <-
+function(x, intercept = FALSE, ...) {
+	formList <- split_formula_by_bar(formula(x))
+	formList <- lapply(lapply(formList, terms.formula, data = model.frame(x)),
+		formula)
+	oneform <- length(formList) == 1L
+	z <- lapply(formList, getAllTerms, intercept = TRUE)
+	
+	deps <- termdepmat_combine(lapply(z, attr, "deps"))
+	
+	ord <- unlist(lapply(z, attr, "order"))
+	n <- sapply(z, length)
+	if(length(z) > 1L) ord[-j] <- ord[-(j <- seq_len(n[1L]))] + n[1L]
+	zz <- unlist(z)
+	interceptIdx <- zz == "(Intercept)"
+	offsetIdx <- match(zz, unique(unlist(lapply(z, attr,"offset"))), nomatch = 0) != 0
+	termIdx <- !(offsetIdx | interceptIdx)
+	
+	if(!oneform && n[2L] != 0L) {
+		i.phi <- -seq.int(n[1L])
+		zz[i.phi] <- paste("(phi)", zz[i.phi], sep = "_")
+	}
+	dimnames(deps) <- list(zz[termIdx], zz[termIdx])
+	
+	ret <- if(!intercept) zz[!interceptIdx] else zz
+	if(any(offsetIdx)) attr(ret, "offset") <- zz[offsetIdx]
+	attr(ret, "intercept") <- pmin(which(interceptIdx), 1)
+	attr(ret, "interceptLabel") <- zz[interceptIdx]
+	attr(ret, "response") <- attr(z[[1L]], "response")
+	attr(ret, "order") <- if(!intercept) order(ord[!interceptIdx]) else ord
+	attr(ret, "deps") <- deps
+	ret
+}
+
+
+
+
 `getAllTerms.glimML` <- function(x, intercept = FALSE, ...) {
 	ret <- getAllTerms.default(x, intercept = intercept, ...)
 	ttran <- terms.formula(x@random)
@@ -249,7 +298,7 @@ function(x, ...)  {
 function (x, ...) {
 	res <- getAllTerms.default(x, ...) 
 	attr(res, "random") <- .formulaEnv(.~., environment(formula(x)))
-	attr(res, "random.terms") <- deparse(x$Random$formula, control = NULL)[1]
+	attr(res, "random.terms") <- asChar(x$Random$formula)[1L]
 	res
 }
 
@@ -271,7 +320,7 @@ function (x, intercept = FALSE, ...) {
 	wrapfunc <- function(x, func) if(length(x) == 0L) x else paste0(func, "(", x, ")")
 
 	alltermlist <- lapply(formlist, function(x, intercept) {
-		func <- deparse(x[[1L]], control = NULL)
+		func <- asChar(x[[1L]])
 		at <- getAllTerms(terms(eval(call("~", x[[2L]]))), intercept = intercept)
 		at[] <- wrapfunc(at, func)
 		dn <- wrapfunc(rownames(attr(at, "deps")), func)
@@ -290,49 +339,10 @@ function (x, intercept = FALSE, ...) {
 	retval
 }
 
-`getAllTerms.betareg` <-
-function(x, intercept = FALSE, ...) {
-	f <- formula(x)
-	if(length(f[[3L]]) != 1L && f[[3L]][[1L]] == "|"){
-		f1 <- call("~", f[[2L]], f[[3L]][[2L]])
-		f2 <- call("~", f[[3L]][[3L]])
-	} else {
-		f1 <- f
-		f2 <- NULL
-	}
-	fs <- lapply(lapply(c(f1, f2), terms.formula, data = model.frame(x)),
-		formula)
-	z <- lapply(fs, getAllTerms, intercept = TRUE)
-	
-	deps <- termdepmat_combine(lapply(z, attr, "deps"))
-	
-	ord <- unlist(lapply(z, attr, "order"))
-	n <- sapply(z, length)
-	if(length(z) > 1L) ord[-j] <- ord[-(j <- seq_len(n[1L]))] + n[1L]
-	zz <- unlist(z)
-	Ints <- which(zz == "(Intercept)")
-
-	if(!is.null(f2) && n[2L] != 0L) {
-		i.phi <- -seq.int(n[1L])
-		zz[i.phi] <- paste("(phi)", zz[i.phi], sep = "_")
-	}
-	ret <- if(!intercept) zz[-Ints] else zz
-	dimnames(deps) <- list(zz[-Ints], zz[-Ints])
-
-	attr(ret, "intercept") <- pmin(Ints, 1)
-	attr(ret, "interceptLabel") <- zz[Ints]
-	attr(ret, "response") <- attr(z[[1L]], "response")
-	attr(ret, "order") <- if(!intercept) order(ord[-Ints]) else ord
-	
-	attr(ret, "deps") <- deps
-	ret
-}
-
 `getAllTerms.asreml`  <-
 function(x, intercept = FALSE, ...)
 getAllTerms.terms(terms(formula(x), ...), intercept = intercept)
 
-
 `getAllTerms.cpglmm` <-
 function (x, intercept = FALSE, ...) 
 getAllTerms(x@formula, intercept = intercept)
