@@ -1,11 +1,15 @@
 # Hidden functions
 `.getLogLik` <- function()
-	if(isGeneric("logLik")) .xget("stats4", "logLik") else
-		.xget("stats", "logLik")
-	#if(isGeneric("logLik")) stats4:::logLik else
-	#	stats::logLik
+	if(isGeneric("logLik")) getFrom("stats4", "logLik") else
+		getFrom("stats", "logLik")
+	
+fixLogLik <-
+function(ll, object) {
+	if(is.null(attr(ll, "nall")) && is.null(attr(ll, "nobs")))
+		attr(ll, "nobs") <- nobs(object)
+	ll	
+}
 
-		
 `.getLik` <- function(x) {
     if(isGEE(x)) {
 		logLik <- quasiLik
@@ -17,6 +21,7 @@
 	list(logLik = logLik, name = lLName)
 }
 
+
 `.getRank` <- function(rank = NULL, rank.args = NULL, object = NULL, ...) {
 	rank.args <- c(rank.args, list(...))
 
@@ -24,9 +29,9 @@
 		x <- NULL # just not to annoy R check
 		IC <- as.function(c(alist(x =, do.call("AICc", list(x)))))
 		attr(IC, "call") <- call("AICc", as.name("x"))
-		class(IC) <- c("function", "ICWithCall")
+		class(IC) <- c("function", "rankFunction")
 		return(IC)
-	} else if(inherits(rank, "ICWithCall") && length(rank.args) == 0L) {
+	} else if(inherits(rank, "rankFunction") && length(rank.args) == 0L) {
 		return(rank)
 	}
 
@@ -47,7 +52,7 @@
 	}
 
 	attr(IC, "call") <- ICCall
-	class(IC) <- c("function", "ICWithCall")
+	class(IC) <- c("function", "rankFunction")
 	IC
 }
 
@@ -177,9 +182,32 @@ function(x, split = ":",
 }
 
 getResponseFormula <-
-function(f)
+function(f) {
+	f <- formula(f)
 	if((length(f) == 2L) || (is.call(f[[2L]]) && f[[2L]][[1L]] == "~"))
 		0 else f[[2L]]
+}
+
+get.response <-
+function(x, ...)
+UseMethod("get.response")
+
+get.response.default <-
+function(x, ...) {
+	model.frame(x)[, asChar(getResponseFormula(x), control = NULL)]
+}
+
+get.response.lm <-
+function(x, ...) 
+if((family(x)$family != "binomial") && !is.null(x$y)) x$y else NextMethod()
+# NOTE: for 'binomial' 'y' is a vector not nmatrix2  
+
+get.response.averaging <-
+function(x, ...) {
+	if(is.null(attr(x, "modelList")))
+		stop("'x' has no model list")
+	get.response(attr(x, "modelList")[[1L]])
+}
 
 
 #Tries to find out whether the models are fitted to the same data
@@ -192,12 +220,16 @@ function(f)
 
 	responses <- lapply(models, function(x) getResponseFormula(formula(x)))
 
- 	if(!all(vapply(responses[-1L], "==", logical(1L), responses[[1L]]))) {
+ 	if(!all(vapply(responses[-1L], "==", FALSE, responses[[1L]]))) {
 		err("response differs between models")
 		res <- FALSE
 	}
 
-	datas <- lapply(models, function(x) .getCall(x)$data)
+	#datas <- lapply(models, function(x) get_call(x)$data)
+	# XXX: need to compare deparse'd 'datas' due to ..1 bug(?) in which dotted
+	#  arguments (..1 etc) passed by lapply are not "identical"
+	datas <- vapply(lapply(models, function(x) get_call(x)$data), asChar, "")
+		
 	# XXX: when using only 'nobs' - seems to be evaluated first outside of MuMIn
 	# namespace which e.g. gives an error in glmmML - the glmmML::nobs method 
 	# is faulty.
@@ -214,7 +246,7 @@ function(f)
 
 
 .checkNaAction <-
-function(x, cl = getCall(x),
+function(x, cl = get_call(x),
 		 naomi = c("na.omit", "na.exclude"), what = "model") {
 	naact <- NA_character_
 	msg <- NA_character_
@@ -376,12 +408,12 @@ function(models, withModel = FALSE, withFamily = TRUE,
 	}
 
 	if(withArguments) {
-		cl <- lapply(models, .getCall)
+		cl <- lapply(models, get_call)
 		haveNoCall <-  vapply(cl, is.null, FALSE)
 		cl[haveNoCall] <- lapply(cl[haveNoCall], function(x) call("none", formula = NA))
  		arg <- lapply(cl, function(x) sapply(x[-1L], function(argval)
 			switch(mode(argval), character = , logical = argval,
-			numeric = signif(argval, 3L), deparse(argval, control = NULL, nlines = 1L))))
+			numeric = signif(argval, 3L), asChar(argval))))
 		arg <- rbindDataFrameList(lapply(lapply(arg, t), as.data.frame))
 		arg <- cbind(class = as.factor(sapply(lapply(models, class), "[", 1L)),
 			arg[, !(colnames(arg) %in% remove.cols), drop = FALSE])
@@ -405,7 +437,6 @@ function(models, withModel = FALSE, withFamily = TRUE,
 	ret
 }
 
-
 family2char <-
 function(x, fam = x$family, link = x$link) {
 	if(nchar(fam) > 17L && (substr(fam, 1L, 17) == "Negative Binomial")) {
@@ -415,3 +446,44 @@ function(x, fam = x$family, link = x$link) {
 		paste0(fam, "(", link, ")")
 	}
 }
+
+`commonCallStr` <-
+function(models, calls = lapply(models, get_call)) {
+	
+	x <- lapply(calls, as.list)
+	alln <- unique(unlist(lapply(x, names)))
+	uniq <- vector("list", length(alln))
+	names(uniq) <- alln
+	uniq[[1L]] <- lapply(x, "[[", 1L)
+	for(i in alln[-1]) uniq[[i]] <- lapply(x, "[[", i)
+	uniq <- lapply(uniq, unique)
+	nu <- sapply(uniq, length)
+	strvarious <- "<*>"
+	rval <- lapply(uniq, '[[', 1L)
+	j <- sapply(rval, inherits, "formula")
+	for(i in which(j))
+		rval[[i]] <- call("~", getResponseFormula(rval[[i]]), as.name(sprintf("__%d-rhsform__", nu[i])))
+	j <- nu > 1 & !j
+	rval[j] <- paste("<", nu[j], " unique values>", sep = "")
+	if(nu[1L] > 1) rval[[1L]] <- paste(sapply(uniq[[1L]], asChar), collapse = "|")
+	rval <- paste(rval[[1L]], "(", paste(names(rval[-1L]), "=", rval[-1L], collapse = ", "), ")", sep = "")
+	rval <- sub("`__(\\d+)-rhsform__`", "<\\1 unique rhs>", rval, perl = TRUE)
+	rval
+	
+}
+
+updateDeps <-
+function(expr, deps) {
+	ret <- list()
+	env <- sys.frame(sys.nframe())
+	expr <- .exprapply(expr, "dc", function(z) {
+		v <- vapply(as.list(z[-1L]), asChar, "")
+		n <- length(v)
+		k <- match(v, colnames(deps))
+		for(i in 2L:n) deps[k[1L:(i - 1L)], k[i]] <- TRUE
+		assign("deps", deps, envir = env, inherits = FALSE)
+		TRUE
+	})
+	list(deps = deps, expr = expr)
+}
+
