@@ -35,7 +35,7 @@ budworm$SF <- cbind(numdead = budworm$numdead, numalive = 20 - budworm$numdead)
 	mod <- glm(SF ~ sex*ldose, data = dat2, family = "binomial")
 	#mod <- glm(SF ~ sex*ldose, data = budworm, family = "binomial", trace=F)
 	print(dd <- dredge(mod, rank = "QAIC", chat = summary(budworm.lg)$dispersion))
-	gm <- get.models(dd, family="binomial")
+	gm <- get.models(dd, subset = NA, family = "binomial")
 	#print(sys.frames())
 	summary(model.avg(gm))
 })(dat))(budworm)
